#!/usr/bin/python
from collections import defaultdict
from pylcdsysinfo import LCDSysInfo, TextLines, BackgroundColours, TextColours, TextAlignment, COL2LEFT

import re
import sys
import time
import math
import json
import requests
import datetime

################################################################################
# CUSTOMIZABLE VARS
################################################################################

debug           = 0         # 0 - No output, 1 - Basic, 2 - JSON
update_interval = 10        # number of seconds to wait before updating display

# LCD
brightness      = 100

# M/Monit
green_hosts     = 0         # output green hosts
retries         = 10        # HTTP retries before failing and exiting

host            = "https://MMONIT_URL:PORT"
username        = "USER"
password        = "PASS"

################################################################################
# FIXED VARS
################################################################################

colors          = defaultdict(int)
colors[0]       = 'red'
colors[1]       = 'orange'
colors[2]       = 'green'
colors[3]       = 'gray'

requests.adapters.DEFAULT_RETRIES = retries

################################################################################
# SUBROUTINES
################################################################################

def mmonit_loop(s, bg=None):
        floor            = math.floor  # minor optimization
        previous_clock   = ""
        previous_message = ""

        d = LCDSysInfo()
        d.dim_when_idle(False)
        d.set_brightness(brightness)
        d.save_brightness(brightness, 255)

        d.display_icon(0, 180)


        while 1:
                start_time     = time.time()
                line_num       = 3
                clear_line     = 4
                column         = 0


                # Get Date and print Clock at left upper corner
                localtime  = time.localtime()
                clock_str  = time.strftime("%H:%M", localtime)

                # Minute changed? Clear screen and show clock
                if clock_str != previous_clock:
                        d.display_icon(0, 180)
                        d.display_text_on_line(0, clock_str, False, None, TextColours.WHITE)
                        previous_clock = clock_str

                url = host + "/status/hosts/list"
                r = s.post(url, verify=False)

                if (r.status_code == requests.codes.ok):
                        commit_data = r.json()

			# Debug JSON
                        if debug == 2:
                                print r.text
                                print s.cookies
                                print commit_data.keys()
                                print commit_data[u'totalRecords']
                                print commit_data[u'records']


                        # Order by led colors
                        for i in sorted(commit_data['records'],key=lambda x:x['led']):
                                message = ''
                                icon    = 0
                                ipos    = (line_num - 1) * 8
                                led     = colors[i['led']]

                                # Only show 1.st word of hostname - Usually its FQDN
                                hostname = re.sub(r'\..*', '', i['hostname'])

#                                NOT USED - Too many letters
#                                status = i['status'].replace(" out of ", "/")
#                                status = re.sub(r' .*', '', i['status'])


                                # Errors:
				#      0=red, 1=orange, 2=green, 3=gray 
                                if led == 'red':
                                        message = hostname
                                        color   = TextColours.RED
                                        icon    = 5
                                elif led == 'orange':
                                        message = hostname
                                        color = TextColours.ORANGE
                                        icon    = 19
                                elif led == 'green':
                                        if green_hosts:
                                                message = hostname
                                                color = TextColours.GREEN
                                                icon    = 6
                                elif led == 'gray':
                                        message = hostname
                                        color = TextColours.GREY
                                        icon    = 42

				# Debug Records
                                if message and debug == 1:
                                        print 'Message:' + message
                                        print 'Color:' + led
                                        print 'Column:' + str(column)
                                        print 'Line Number:' + str(line_num)

				# Cycle through columns
                                if column % 2:
                                        # Print full line saved + now
                                        if message:
                                                message = previous_message + "\t" + message
                                                d.display_text_on_line(line_num, message, True, (TextAlignment.LEFT, TextAlignment.LEFT), color)
                                                d.display_icon(ipos, icon)
                                                d.display_icon(ipos+4, icon)

                                                line_num = line_num + 1
                                                clear_line = clear_line * 2
                                                if line_num > 6:
                                                        line_num   = 3
                                                        clear_line = 4
                                                previous_message = ''
                                else:
                                        # Don't print... save
                                        if message:
                                                previous_message = message
                                                previous_color   = color
                                                previous_ipos    = ipos
                                                previous_icon    = icon

                                column = column + 1

                        # Single Message per line
                        if previous_message:
                                d.display_text_on_line(line_num, previous_message, True, TextAlignment.LEFT, previous_color)
                                d.display_icon(previous_ipos, previous_icon)

                if debug:
                        print "Loop"

                # Work out when to wake up for the next round/whole (non-fractional) time
                future_time = floor(start_time) + update_interval
                sleep_time = future_time - start_time
                time.sleep(sleep_time)

################################################################################
# MAIN CODE
################################################################################

def main(argv=None):
        if argv is None:
                argv = sys.argv

        data = { "z_username":username, "z_password":password }

        # Initiate Session
        s = requests.Session()

        url = host + "/index.csp"
        r = s.get(url, verify=False)

        # Add Authentication
        s.headers.update(data)

        url = host + "/z_security_check"
        r = s.post(url,data=data, verify=False)

        # LOOP
        mmonit_loop(s)

        return 0

if __name__ == "__main__":
        sys.exit(main())
